/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "pidl.h"

LPITEMIDLIST PIDLAppend(LPCITEMIDLIST pidlBase, LPCITEMIDLIST pidlAdd, LPMALLOC pMalloc)
{
	if(pidlBase == NULL) {
        return PIDLCopy(pidlAdd);
	}
	if(pidlAdd == NULL) {
        return PIDLCopy(pidlBase);
	}
    
    LPITEMIDLIST pidlNew;

    UINT cb1 = PIDLGetSize(pidlBase) - sizeof(pidlBase->mkid.cb);
    UINT cb2 = PIDLGetSize(pidlAdd);

    pidlNew = (LPITEMIDLIST)pMalloc->Alloc(cb1 + cb2);
    if(pidlNew) {
        CopyMemory(pidlNew, pidlBase, cb1);
        CopyMemory(((LPSTR)pidlNew) + cb1, pidlAdd, cb2);
    }
    return pidlNew;
}

/**
 * PIDLGetSize
 * Computes the total size of an item ID list.
 */
UINT PIDLGetSize(LPCITEMIDLIST pidl)
{
    UINT cbTotal = 0;
    if(pidl) {
        cbTotal += sizeof(pidl->mkid.cb);    // Null terminator
        while(pidl) {
            cbTotal += pidl->mkid.cb;
            pidl = PIDLGetNextItem(pidl);
        }
    }
    return cbTotal;
}

LPITEMIDLIST PIDLCopy(LPCITEMIDLIST pidl, UINT cb)
{
    LPMALLOC pMalloc;
	if(!SUCCEEDED(SHGetMalloc(&pMalloc))) {
		return NULL;
	}

	// Calculate size of list.
	if(cb == -1) {
	    cb = PIDLGetSize(pidl);
	}

    LPITEMIDLIST pidlRet = (LPITEMIDLIST)pMalloc->Alloc(cb);
	if(pidlRet) {
        CopyMemory(pidlRet, pidl, cb);
	}

    pMalloc->Release();

    return pidlRet;
}

LPITEMIDLIST PIDLCopyFirst(LPITEMIDLIST pidl)
{
	USHORT nLen;
	LPITEMIDLIST pidlNew;
    LPMALLOC pMalloc;
	if(!SUCCEEDED(SHGetMalloc(&pMalloc))) {
		return NULL;
	}

	nLen = pidl->mkid.cb;
	if(nLen == 0) {
		return NULL;
	}

	pidlNew = (LPITEMIDLIST)pMalloc->Alloc(nLen + sizeof(USHORT));
	if(pidlNew == NULL) {
		return NULL;
	}

	CopyMemory(pidlNew, pidl, nLen);
	*((USHORT*)(((LPBYTE)pidlNew) + nLen)) = 0;

	return pidlNew;
}

/**
 * PIDLGetItemCount
 * Computes the number of item IDs in an item ID list.
 */
UINT PIDLGetItemCount(LPCITEMIDLIST pidl)
{
	UINT nCount;
	nCount = 0;
	while(pidl->mkid.cb != 0) {
		pidl = PIDLGetNextItem(pidl);
		nCount++;
	}
	return nCount;
}

/**
 * PIDLGetNextItem
 * Returns the next item in the item ID list.
 */
LPITEMIDLIST PIDLGetNextItem(LPCITEMIDLIST pidl)
{
	if(!pidl) {
		return NULL;
	}

	USHORT nLen = pidl->mkid.cb;

	if(nLen == 0) {
		return NULL;
	}

	return (LPITEMIDLIST)(((LPBYTE)pidl) + nLen);
}

LPBYTE PIDLGetPos(LPCITEMIDLIST pidl, int nPos)
{
	if(!pidl) {
		return 0;
	}
	int nCount = 0;
	
	BYTE * pCur = (BYTE *)pidl;
	while(((LPCITEMIDLIST) pCur)->mkid.cb) {
		if(nCount == nPos) {
			return pCur;
		}
		nCount++;
		pCur += ((LPCITEMIDLIST)pCur)->mkid.cb;	// + sizeof(pidl->mkid.cb);
	}
	if(nCount == nPos) {
		return pCur;
	}
	return NULL;
}

void PIDLFreeArray(LPITEMIDLIST *pidlArray)
{
	if(!pidlArray) {
		return;
	}

	int iSize = _msize(pidlArray) / sizeof(LPITEMIDLIST);

	for(int i = 0; i < iSize; i++) {
		free(pidlArray[i]);
	}
	free(pidlArray);
}

/**
 * PIDLGetParent
 * Returns the parent of the supplied item.
 */
/*
LPITEMIDLIST PIDLGetParent(LPITEMIDLIST pidlFile)
{
	if(pidlFile == NULL) {
		return NULL;
	}

	LPMALLOC pMalloc;
	if(!SUCCEEDED(SHGetMalloc(&pMalloc))) {
		return NULL;
	}

	LPITEMIDLIST pidl = pidlFile;
	UINT cbTotal = 0;
	UINT cbLast = 0;
	while(pidl->mkid.cb) {
		cbTotal += cbLast;
		cbLast = pidl->mkid.cb;
		pidl = (LPITEMIDLIST)((BYTE*)pidl + cbLast);
	}
	LPITEMIDLIST pidlParent = (LPITEMIDLIST)pMalloc->Alloc(cbTotal + 2);
	ZeroMemory(pidlParent + cbTotal, 2);
	CopyMemory(pidlParent, pidlFile, cbTotal);

	pMalloc->Release();
	return pidlParent;
}
*/

/**
 * PIDLCopyLast
 * Returns the last item in the item ID list.
 */
/*
LPITEMIDLIST PIDLCopyLast(LPITEMIDLIST pidl)
{
	if(pidl == NULL) {
		return NULL;
	}

	LPITEMIDLIST pidlLoop = pidl;
	UINT nCount = PIDLGetItemCount(pidl);
	while(--nCount) {
		pidlLoop = PIDLGetNextItem(pidlLoop);
	}
	return PIDLCopy(pidlLoop);
}
*/
